﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace HabboTraxDEBESTE2
{
    /*
     * Todo:
     *  - Fix auto stop at end of song
     */
    public partial class TraxMachineForm : Form
    {
        private PictureBox[,] currentTraxImages;// = new PictureBox[24, 4];
        private Panel[,] currentTraxInbetweenColors;
        private int traxEditorImageSize = 40;
        private float traxEditorInbetweenHeightPercentage = 0.66f;
        private int traxEditorInbetweenOverlap = 7;
        private int traxEditorImageMargins = 3;
        private int maxNumberOfCds = 4;
        private TraxSong song;
        private Color DefaultBackgroundColor = Color.FromArgb(89, 117, 128);
        private Color DefaultSelectionBackgroundColor = Color.FromArgb(85, 85, 85);
        private Trax draggingTrax;
        private Rectangle dragBoxFromMouseDown;
        private Point screenOffset;
        private string filename;
        private bool changedSinceLastSave = false;
        private bool ChangedSinceLastSave
        {
            get { return changedSinceLastSave; }
            set
            {
                changedSinceLastSave = value;
                UpdateFormCaption();
            }
        }
        private bool playing = false;
        private bool Playing
        {
            get { return playing; }
            set
            {
                playing = value;
                //Make this operation thread independent
                btn_Play.Invoke(new UpdatePlayButtonTextEventHandler(UpdatePlayButtonText));
                currentPosIndicator.Invoke(new UpdateCurrentPosIndicatorVisibilityEventHandler(UpdateCurrentPosIndicatorVisibility));
            }
        }

        private delegate void UpdatePlayButtonTextEventHandler();
        private void UpdatePlayButtonText()
        {
            btn_Play.Text = playing ? "Stop" : "Play";
        }

        private delegate void UpdateCurrentPosIndicatorVisibilityEventHandler();
        private void UpdateCurrentPosIndicatorVisibility()
        {
            currentPosIndicator.Visible = playing;
        }


        //Currently mounted CDs
        private Dictionary<string, int> cdPlayerNrs;

        //Remembers what player the cd was mounted in, so we know what colors to use
        private Dictionary<string, int> cdMountedAsPlayer;
        private Player player;
        private PlayingFrameSquare currentPosIndicator;


        public TraxMachineForm()
        {
            player = new Player();
            cdPlayerNrs = new Dictionary<string,int>();
            cdMountedAsPlayer = new Dictionary<string, int>();
            song = new TraxSong();
            song.FinishedPlaying += new EventHandler(SongFinishedPlaying);
            song.NextPageStarts = NextSongPage;
            song.NextSlotPlays = NextSlotPlays;
            filename = string.Empty;
            InitializeComponent();
            this.AllowDrop = true;

            ckl_chosenCDs.Items.AddRange(TraxCDs);

            tlp_CD1.Visible = tlp_CD2.Visible = tlp_CD3.Visible = tlp_CD1Header.Visible = tlp_CD2Header.Visible = tlp_CD3Header.Visible = false;

            Console.WriteLine("test");
            RefreshPictureBoxes();
            RefreshGrid();
            
        }

        #region Refresh

        private void RefreshPictureBoxes()
        {
            pnl_currentTrax.HorizontalScroll.Value = 0;
            pnl_currentTrax.SuspendLayout();
            this.SuspendLayout();
            //Dispose old picture boxes:
            if (currentTraxImages != null)
                for (int i = 0; i < currentTraxImages.GetLength(0); i++)
                    for (int j = 0; j < currentTraxImages.GetLength(1); j++)
                        currentTraxImages[i, j].Dispose();
            if (currentTraxInbetweenColors != null)
                for (int i = 0; i < currentTraxInbetweenColors.GetLength(0); i++)
                    for (int j = 0; j < currentTraxInbetweenColors.GetLength(1); j++)
                        currentTraxInbetweenColors[i, j].Dispose();


            currentTraxImages = new PictureBox[song.GetPage(0).Length, song.GetPage(0).Height];
            currentTraxInbetweenColors = new Panel[song.GetPage(0).Length - 1, song.GetPage(0).Height];
            for (int i = 0; i < currentTraxImages.GetLength(0); i++)
            {
                for (int j = 0; j < currentTraxImages.GetLength(1); j++)
                {
                    //Add the picturebox
                    currentTraxImages[i, j] = new PictureBox();
                    currentTraxImages[i, j].BorderStyle = BorderStyle.FixedSingle;
                    currentTraxImages[i, j].Parent = pnl_currentTrax;
                    currentTraxImages[i, j].Location = new Point(i * (traxEditorImageSize + traxEditorImageMargins) + traxEditorImageMargins, j * (traxEditorImageSize + traxEditorImageMargins) + traxEditorImageMargins);
                    currentTraxImages[i, j].AllowDrop = true;
                    currentTraxImages[i, j].DragDrop += new DragEventHandler(TraxImages_DragDrop);
                    currentTraxImages[i, j].DragEnter += new DragEventHandler(TraxImages_DragEnter);
                    //currentTraxImages[i, j].DragOver += new DragEventHandler(TraxMachineForm_DragOver);
                    currentTraxImages[i, j].MouseClick += new MouseEventHandler(TraxImages_Click);
                    currentTraxImages[i, j].Size = new Size(traxEditorImageSize, traxEditorImageSize);
                    currentTraxImages[i, j].Margin = new Padding(0);
                    currentTraxImages[i, j].Tag = new Tuple<int, int>(i, j); //Save the position of the pictureBox relative to the grid

                    if (i < currentTraxInbetweenColors.GetLength(0))
                    {
                        currentTraxInbetweenColors[i, j] = new Panel();
                        currentTraxInbetweenColors[i, j].Parent = pnl_currentTrax;
                        currentTraxInbetweenColors[i, j].Location = new Point(i * (traxEditorImageSize + traxEditorImageMargins) + traxEditorImageSize - traxEditorInbetweenOverlap + traxEditorImageMargins,
                            (int)(j * (traxEditorImageSize + traxEditorImageMargins) + (.25f * traxEditorInbetweenHeightPercentage) * (traxEditorImageSize+traxEditorImageMargins)));
                        currentTraxInbetweenColors[i, j].Margin = new Padding(0);
                        currentTraxInbetweenColors[i, j].Padding = new Padding(0);
                        currentTraxInbetweenColors[i, j].AllowDrop = false;
                        currentTraxInbetweenColors[i, j].Size = new Size(traxEditorImageMargins + 2 * traxEditorInbetweenOverlap, (int)(traxEditorInbetweenHeightPercentage * traxEditorImageSize));
                        currentTraxInbetweenColors[i, j].BringToFront();
                    }
                }
            }
            pnl_currentTrax.ResumeLayout();
            this.ResumeLayout();
        }

        private void RefreshGrid()
        {
            int w = currentTraxImages.GetLength(0);
            int h = currentTraxImages.GetLength(1);

            //Reset all images
            for (int i = 0; i < w; i++)
                for (int j = 0; j < h; j++)
                {
                    currentTraxImages[i, j].Image = null;
                    currentTraxImages[i, j].BorderStyle = BorderStyle.FixedSingle;
                }
            for (int i = 0; i < currentTraxInbetweenColors.GetLength(0); i++)
                for (int j = 0; j < currentTraxInbetweenColors.GetLength(1); j++)
                    currentTraxInbetweenColors[i, j].Visible = false;

            for (int i = 0; i < w; i++)
                for (int j = 0; j < h; j++)
                {
                    if (song.CurrentTraxPage().GetTrax(i, j) != null)
                    {
                        Trax currentTrax = song.CurrentTraxPage().GetTrax(i, j);
                        int player = 1;
                        if (cdMountedAsPlayer.ContainsKey(currentTrax.CD))
                            player = cdMountedAsPlayer[currentTrax.CD];
                        currentTraxImages[i, j].Image = ImageScaler.Scale(traxPartImages[player][currentTrax.TraxNr - 1], 
                            new Size(traxEditorImageSize, traxEditorImageSize));
                        currentTraxImages[i, j].BorderStyle = BorderStyle.None;
                        if (song.CurrentTraxPage().Length > i && song.CurrentTraxPage().GetTrax(i + 1, j) != null &&
                            song.CurrentTraxPage().GetTrax(i, j).Code == song.CurrentTraxPage().GetTrax(i + 1, j).Code &&
                            song.CurrentTraxPage().HasNextPart(i, j))
                        {
                            currentTraxInbetweenColors[i, j].BackColor = traxColors[player - 1];
                            currentTraxInbetweenColors[i, j].Visible = true;
                            currentTraxInbetweenColors[i, j].BringToFront();
                        }

                    }
                }
        }
        #endregion

        void TraxImages_Click(object sender, MouseEventArgs e)
        {
            PictureBox pcb = sender as PictureBox;
            if (pcb == null)
                return;
            Tuple<int, int> location = pcb.Tag as Tuple<int, int>;
            if (location == null)
                return;
            if (e.Button.Equals(MouseButtons.Left))
            {
                Trax clickedTrax = song.CurrentTraxPage().GetTrax(location.Item1, location.Item2);
                if (clickedTrax != null)
                    player.PlaySound(clickedTrax);
            }
            if (e.Button.Equals(MouseButtons.Right))
            {
                song.CurrentTraxPage().RemoveTrax(location.Item1, location.Item2);
                ChangedSinceLastSave = true;
                RefreshGrid();
            }
        }

        #region SelectingCDs
        private void ckl_chosenCDs_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            string cd = ckl_chosenCDs.Items[e.Index] as string;
            if (e.NewValue == CheckState.Checked)
            {
                int nrOfCdChecked = ckl_chosenCDs.CheckedItems.Count + 1;
                if (nrOfCdChecked > maxNumberOfCds)
                {
                    MessageBox.Show(this, string.Format("Maximaal {0} CDs!",maxNumberOfCds),
                        string.Format("Er mogen maximaal {0} CDs geselecteerd worden", maxNumberOfCds),
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    e.NewValue = CheckState.Unchecked;
                    return;
                }
                int firstFreeCDPlayer = 0; //4 is highest...
                for (int i = 1; i < 5; i++)
                {
                    if (!cdPlayerNrs.Values.Contains(i))
                    {
                        firstFreeCDPlayer = i;
                        break;
                    }
                }
                SelectCD(firstFreeCDPlayer, cd);

            }
            else
            {
                RemoveCD(cd);
            }
        }

        private void SelectCD(int nrToLoad, string cd)
        {
            
            switch (nrToLoad)
            {
                case 1:
                    SelectCD(cd, nrToLoad, lbl_CD1, pcb_CD1, tlp_CD1Header, tlp_CD1);
                    break;
                case 2:
                    SelectCD(cd, nrToLoad, lbl_CD2, pcb_CD2, tlp_CD2Header, tlp_CD2);
                    break;
                case 3:
                    SelectCD(cd, nrToLoad, lbl_CD3, pcb_CD3, tlp_CD3Header, tlp_CD3);
                    break;
                case 4:
                    SelectCD(cd, nrToLoad, lbl_CD4, pcb_CD4, tlp_CD4Header, tlp_CD4);
                    break;
                default:
                    //Error
                    break;
            }

        }

        private void SelectCD(string cd, int nrToLoad, Label ToPutNameIn, PictureBox ToPutPictureIn, TableLayoutPanel header, TableLayoutPanel control)
        {
            if (cdPlayerNrs.ContainsKey(cd))
                RemoveCD(cd);
            cdPlayerNrs.Add(cd, nrToLoad);
            ToPutNameIn.Text = cd;
            string diskNumber = GetDiskNumber(cd);
            if (!cdMountedAsPlayer.ContainsKey(diskNumber))
                cdMountedAsPlayer.Add(diskNumber, nrToLoad);
            cdMountedAsPlayer[diskNumber] = nrToLoad;
            ToPutPictureIn.Image = ImageScaler.Scale(diskImages.ContainsKey(diskNumber) ? diskImages[diskNumber] : null, ToPutPictureIn.Size);
            Trax[] traxToLoadForCD = new Trax[9];
            for (int i = 0; i < traxToLoadForCD.Length; i++)
                traxToLoadForCD[i] = new Trax(diskNumber, i + 1);

            //Trax[] currentTrax = player.TraxNeeded.Concat(traxToLoadForCD).ToArray()
            player.AddTraxNeeded(traxToLoadForCD);

            foreach (Trax tr in traxToLoadForCD)
            {
                PictureBox pictureBox = new PictureBox();
                pictureBox.Image = ImageScaler.Scale(traxPartImages[nrToLoad][tr.TraxNr - 1], pictureBox.Size);
                pictureBox.Tag = tr;
                control.Controls.Add(pictureBox, (tr.TraxNr - 1) % 3, (tr.TraxNr - 1) / 3);
                pictureBox.MouseClick += new MouseEventHandler(pictureBox_MouseClick);
                pictureBox.MouseDown += new MouseEventHandler(pictureBox_MouseDown);
                pictureBox.MouseUp += new MouseEventHandler(pictureBox_MouseUp);
                pictureBox.MouseMove += new MouseEventHandler(pictureBox_MouseMove);
                pictureBox.QueryContinueDrag += new QueryContinueDragEventHandler(pictureBox_QueryContinueDrag);
                
            }

            header.Visible = true;
            control.Visible = true;
        }

        private void RemoveCD(string cd)
        {
            int nrToLoad = -1;
            if (cdPlayerNrs.ContainsKey(cd))
                nrToLoad = cdPlayerNrs[cd];
            switch (nrToLoad)
            {
                case 1:
                    RemoveCD(cd, lbl_CD1, pcb_CD1, tlp_CD1Header, tlp_CD1);
                    break;
                case 2:
                    RemoveCD(cd, lbl_CD2, pcb_CD2, tlp_CD2Header, tlp_CD2);
                    break;
                case 3:
                    RemoveCD(cd, lbl_CD3, pcb_CD3, tlp_CD3Header, tlp_CD3);
                    break;
                case 4:
                    RemoveCD(cd, lbl_CD4, pcb_CD4, tlp_CD4Header, tlp_CD4);
                    break;
                default:
                    //Error
                    break;
            }
        }

        private void RemoveCD(string cd, Label toPutNameIn, PictureBox toPutPictureIn, TableLayoutPanel header, TableLayoutPanel control)
        {
            if (cdPlayerNrs.ContainsKey(cd))
                cdPlayerNrs.Remove(cd);
            ckl_chosenCDs.SetItemChecked(ckl_chosenCDs.Items.IndexOf(cd), false);
            toPutNameIn.Text = string.Empty;
            toPutPictureIn.Image = null;
            header.Visible = false;
            control.Visible = false;
            foreach (Control pcb in control.Controls)
            {
                pcb.Dispose();
            }
            control.Controls.Clear();
        }

        private string GetDiskNumber(string cd)
        {
            int diskNumber = -1;
            for (int i = 0; i < TraxCDs.Length; i++)
                if (TraxCDs[i] == cd)
                    diskNumber = i + 1;
            if (diskNumber == -1)
                throw new KeyNotFoundException("Die CD kunnen we niet vinden");
            return string.Format("{0:00}", diskNumber);
        }
        #endregion

        void pictureBox_MouseClick(object sender, MouseEventArgs e)
        {
            PictureBox box = sender as PictureBox;
            if (box != null)
                player.PlaySound(box.Tag as Trax);
        }

        #region DragDrop
        void pictureBox_MouseDown(object sender, MouseEventArgs e)
        {
            PictureBox box = sender as PictureBox;
            if (box != null)
                draggingTrax = box.Tag as Trax;
            else
                return;

            //If a picturebox with Trax was clicked, start dragging
            if (draggingTrax != null)
            {
                Size dragSize = SystemInformation.DragSize;
                dragBoxFromMouseDown = new Rectangle(new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
            }
            else
                dragBoxFromMouseDown = Rectangle.Empty;
        }

        void pictureBox_MouseUp(object sender, MouseEventArgs e)
        {
            //Reset the dragging
            dragBoxFromMouseDown = Rectangle.Empty;
            draggingTrax = null;
        }

        void pictureBox_MouseMove(object sender, MouseEventArgs e)
        {
            //If the mouse moves outside the rectangle, start dragging!
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                if (dragBoxFromMouseDown != Rectangle.Empty &&
                    !dragBoxFromMouseDown.Contains(e.X, e.Y))
                {
                    // The screenOffset is used to account for any desktop bands 
                    // that may be at the top or left side of the screen when 
                    // determining when to cancel the drag drop operation.
                    screenOffset = SystemInformation.WorkingArea.Location;

                    // Proceed with the drag-and-drop, passing in the list item.
                    PictureBox box = sender as PictureBox;
                    DragDropEffects dropEffect = box.DoDragDrop(draggingTrax, DragDropEffects.All);
                }
            }
        }

        void pictureBox_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //Cancel drag als de muis van het form af beweeg.
            PictureBox box = sender as PictureBox;
            if (box != null)
            {
                Form f = box.FindForm();
                if (((Control.MousePosition.X - screenOffset.X) < f.DesktopBounds.Left) ||
                    ((Control.MousePosition.X - screenOffset.X) > f.DesktopBounds.Right) ||
                    ((Control.MousePosition.Y - screenOffset.Y) < f.DesktopBounds.Top) ||
                    ((Control.MousePosition.Y - screenOffset.Y) > f.DesktopBounds.Bottom))
                {
                    e.Action = DragAction.Cancel;
                }
            }
        }

        void TraxImages_DragEnter(object sender, DragEventArgs e)
        {
            //Console.WriteLine("Drag enter");
            if (e.Data.GetDataPresent(typeof(Trax)))
            {
                e.Effect = DragDropEffects.Move;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        void TraxImages_DragDrop(object sender, DragEventArgs e)
        {
            Trax trax = e.Data.GetData(typeof(Trax)) as Trax;
            if (trax == null)
                return;
            //player.PlaySound(trax);

            //Add the trax at the current position
            PictureBox pcb = sender as PictureBox;
            if (pcb == null)
                return;
            var location = pcb.Tag as Tuple<int, int>;
            if (location == null)
                return;

            try
            {
                song.CurrentTraxPage().AddTrax(trax, location.Item1, location.Item2, false);
                ChangedSinceLastSave = true;
            }
            catch (TileOccupiedException exc)
            {
                Console.WriteLine(exc.ToString());
                if (MessageBox.Show(this, "Op deze plek staat al een trax, wilt u deze vervangen?", "Er is al een trax", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1) == System.Windows.Forms.DialogResult.Yes)
                {
                    song.CurrentTraxPage().AddTrax(trax, location.Item1, location.Item2, true);
                    ChangedSinceLastSave = true;
                }
            }
            finally
            {
                RefreshGrid();
            }
        }

        #endregion

        #region ButtonEventHandlers

        private void NextSongPage(int pageNr)
        {
            //Do nothing since we haven't implemented using more than one page
        }

        private void NextSlotPlays(int slotNr)
        {
            currentPosIndicator.Invoke(new MovePlayerToPosEventHandler(MovePlayerToPos), slotNr);
        }

        delegate void MovePlayerToPosEventHandler(int pos);
        private void MovePlayerToPos(int pos)
        {
            currentPosIndicator.Visible = false;
            currentPosIndicator.Location = new Point((traxEditorImageSize + traxEditorImageMargins) * pos, 0);
            currentPosIndicator.Visible = true;
            currentPosIndicator.BringToFront();
        }

        void SongFinishedPlaying(object sender, EventArgs e)
        {
            //To work thread independent
            Playing = false;
        }       

        private void btn_Play_Click(object sender, EventArgs e)
        {
            if (playing)
            {
                Stop();
            }
            else
            {
                Play();
            }
        }

        private void Play()
        {
            currentPosIndicator = new PlayingFrameSquare(DefaultSelectionBackgroundColor, traxEditorImageSize,
                (traxEditorImageSize + traxEditorImageMargins) * song.CurrentTraxPage().Height,
                3);
            currentPosIndicator.Visible = false;
            currentPosIndicator.Parent = pnl_currentTrax;
            song.Play(player);
            Playing = true;
        }

        private void Stop()
        {
            song.Stop();
            player.ShutUp();
            Playing = false;
        }



        private void btn_Reset_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, "Weet je zeker dat je wilt resetten? Reset gooit je hele liedje leeg!", "Echt resetten?", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == System.Windows.Forms.DialogResult.Yes)
            {
                ResetSong();
            }
        }

        private void ResetSong()
        {
            song = new TraxSong();
            song.FinishedPlaying += new EventHandler(SongFinishedPlaying);
            song.NextPageStarts = NextSongPage;
            song.NextSlotPlays = NextSlotPlays;
            RefreshPictureBoxes();
            RefreshGrid();
        }

        private void ResetCDs()
        {
            string[] mountedCDs =  cdPlayerNrs.Keys.ToArray();
            foreach (var cd in mountedCDs)
            {
                RemoveCD(cd);
            }
        }

        private void btn_AddSlots_Click(object sender, EventArgs e)
        {
            song.IncreasePageLength();
            RefreshPictureBoxes();
            RefreshGrid();
        }

        private void menuItem1_Click(object sender, EventArgs e)
        {
            try
            {
                DialogResult result = MessageBox.Show(this, "Wilt u de huidige Trax opslaan voordat u een nieuwe aanmaakt?", "Opslaan?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
                if (result == System.Windows.Forms.DialogResult.Cancel)
                    return;
                if (result == DialogResult.Yes)
                {
                    Save(string.Empty);
                }
                ResetSong();
                ResetCDs();
            }
            catch
            {
            }

        }

        private void Save(string filename)
        {
            if (filename == string.Empty || filename == null)
            {
                if (song.Title != string.Empty)
                    saveFileDialog1.FileName = song.Title + ".xml";
                DialogResult save = saveFileDialog1.ShowDialog();
                if (save != System.Windows.Forms.DialogResult.OK)
                    return;
                filename = saveFileDialog1.FileName;
            }
            try
            {
                song.SaveToXml(filename, cdPlayerNrs.Keys.ToArray());
                ChangedSinceLastSave = false;
                UpdateFormCaption();
            }
            catch
            {
                if (MessageBox.Show(this, "Er is iets misgegaan met het opslaan :( Nog eens proberen?", "Fout!", MessageBoxButtons.YesNo, MessageBoxIcon.Error) ==
                    DialogResult.Yes)
                    Save(filename);
            }
        }

        private void UpdateFormCaption()
        {
            string caption = string.Empty;
            bool hasTitle = (filename != string.Empty && filename != null) || (song.Title != string.Empty || song.Title != null);
            if (ChangedSinceLastSave)
                caption += "*";
            if (song.Title != null && song.Title != string.Empty)
                caption += song.Title;
            else if (filename != null && filename != string.Empty)
                caption += System.IO.Path.GetFileName(filename);
            if (hasTitle)
                caption += " - ";
            caption += "Habbo Traxmachine";
            this.Text = caption;
        }

        private void Open(string filename)
        {
            try {
                Trax[] traxToLoad;
                string[] cdsToLoad;
                song = TraxSong.LoadFile(filename, out traxToLoad, out cdsToLoad);
                song.FinishedPlaying += new EventHandler(SongFinishedPlaying);
                song.NextPageStarts = NextSongPage;
                song.NextSlotPlays = NextSlotPlays;
                song.LoadAllTraxIntoPlayer(player);
                int nrToLoad = 0;
                foreach (string cd in cdsToLoad)
                {
                    nrToLoad++;
                    if (nrToLoad <= maxNumberOfCds)
                        SelectCD(nrToLoad, cd);
                }
                player.AddTraxNeeded(traxToLoad);
                txt_Title.Text = song.Title;
                RefreshPictureBoxes();
                RefreshGrid();

            } 
            catch (Exception e)
            {
                Console.WriteLine(e);
                MessageBox.Show(this, "Er is iets misgegaan tijdens het openen", "Fout!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void mnu_Save_Click(object sender, EventArgs e)
        {
            if (filename == string.Empty || filename == null)
            {
                if (song.Title != string.Empty)
                    saveFileDialog1.FileName = song.Title + ".xml";
                DialogResult save = saveFileDialog1.ShowDialog();
                if (save != System.Windows.Forms.DialogResult.OK)
                    return;
                filename = saveFileDialog1.FileName;
            }

            Save(filename);
        }


        private void mnu_SaveAs_Click(object sender, EventArgs e)
        {
            if (filename == string.Empty || filename == null)
                saveFileDialog1.FileName = song.Title + ".xml";
            else
                saveFileDialog1.FileName = filename;
            DialogResult save = saveFileDialog1.ShowDialog();
            if (save != System.Windows.Forms.DialogResult.OK)
                return;
            filename = saveFileDialog1.FileName;
            Save(filename);
        }

        private void mnu_Open_Click(object sender, EventArgs e)
        {
            DialogResult open = openFileDialog1.ShowDialog();
            if (open != System.Windows.Forms.DialogResult.OK)
                return;
            filename = openFileDialog1.FileName;
            Open(filename);
        }


        private void mnu_Exit_Click(object sender, EventArgs e)
        {
            if (changedSinceLastSave)
            {
                DialogResult result = MessageBox.Show(this, "Wilt u de wijzigingen die zijn aangemaakt in deze trax opslaan voordat je het programma sluit?", "Opslaan?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
                if (result == DialogResult.Yes)
                {
                    try
                    {
                        Save(string.Empty);
                        this.Close();
                    }
                    catch
                    {
                        //dont close if saving failed..
                    }
                }
                if (result == DialogResult.No)
                    this.Close();
            }
            else
                this.Close();
        }


#endregion

        #region constants
        private string[] TraxCDs = new string[] { 
            "[01] DJ Fuse’s Duck Funk", "[02] DJ Fuse’s Habbo Theme", "[03] BattleBall thema’s", "[04] Zonnig en warm", "[05] Dark Skies", "[06] Ambient soundset", "[07] Meubi Sounds I",
            "[08] Electronica", "[09] Mysto Magica", "[10] Boyband Sensation", "[11] Spicey Donna", "[12] Underground beats", "[13] Cafe Muzzakh", "[14] Cameron’s Ex", "[15] El Generico",
            "[16] Ferry Nultado", "[17] Jive Sideburns", "[18] Little Tanga Beach", "[19] MnM", "[20] Monkey Paradise", "[21] Snotty Day", "[22] A Day in the Park", "[23] Nature Nightlife",
            "[24] Compu FX", "[25] Party Pack", "[26] Bhangra Mangra", "[27] Rasta Santa’s Pack", "[28] Moshy Metal", "[29] Dancefloor Burners", "[30] Double Peaks", "[31] House Loops",
            "[32] Pianissimo", "[33] Yngvie Van Halen", "[34] Rockin’ Riffs", "[35] Supa Funk", "[36] Bossa Nueva", "[37] Habbowood", "[38] Highway to Habbo", "[39] Pixels on the Water",
            "[40] Iron Maid", "[41] Sympathy for the Coder", "[42] Snouthill Horror", "[43] Silence of the Moderators", "[44] Ghost Story", "[45] Berlin Connection", "[46] Club Sounds III",
            "[47] Loco Electro", "[48] Jackin’ Chicago", "[49] Maximum Minimal", "[50] Nu Skool Breakz", "[51] NYC Beat", "[52] State of Trancehouse", "[53] Jingle Beats", "[54] Rudolph’s Loops",
            "[55] RnB Grooves 1", "[56] RnB Grooves 2", "[57] RnB Grooves 3", "[58] RnB Grooves 4", "[59] RnB Grooves 5", "[60] Valentine 1", "[61] Valentine 2", "[62] Alhambra Trax 1", 
            "[63] Alhambra Trax 2", "[64] Alhambra Trax 3", "[65] Tiki Vol. 1", "[66] Tiki Vol. 2", "[67] Tiki Vol. 3", "[68] EC 1", "[69] EC 2", "[70] EC 3", "[71] Icy Trax", "[72] Country sounds",
            "[73] Birthday Special!"
        };


        /// <summary>
        /// Uses disk number string as key (can be found using GetDiskNumber(string))
        /// </summary>
        private Dictionary<string, Image> diskImages = new Dictionary<string, Image>()
        {
            {"01", Resource1._01},
            {"02", Resource1._02},
            {"03", Resource1._03},
            {"04", Resource1._04},
            {"05", Resource1._05},
            {"06", Resource1._06},
            {"07", Resource1._07},
            {"08", Resource1._08},
            {"09", Resource1._09},
            {"10", Resource1._10},
            {"11", Resource1._11},
            {"12", Resource1._12},
            {"13", Resource1._13},
            {"14", Resource1._14},
            {"15", Resource1._15},
            {"16", Resource1._16},
            {"17", Resource1._17},
            {"18", Resource1._18},
            {"19", Resource1._19},
            {"20", Resource1._20},
            {"21", Resource1._21},
            {"22", Resource1._22},
            {"23", Resource1._23},
            {"24", Resource1._24},
            {"25", Resource1._25},
            {"26", Resource1._26},
            {"27", Resource1._27},
            {"28", Resource1._28},
            {"29", Resource1._29},
            {"30", Resource1._30},
            {"31", Resource1._31},
            {"32", Resource1._32},
            {"33", Resource1._33},
            {"34", Resource1._34},
            {"35", Resource1._35},
            {"36", Resource1._36},
            {"37", Resource1._37},
            {"38", Resource1._38},
            {"39", Resource1._39},
            {"40", Resource1._40},
            {"41", Resource1._41},
            {"42", Resource1._42},
            {"43", Resource1._43},
            {"44", Resource1._44},
            {"45", Resource1._45},
            {"46", Resource1._46},
            {"47", Resource1._47},
            {"48", Resource1._48},
            {"49", Resource1._49},
            {"50", Resource1._50},
            {"51", Resource1._51},
            {"52", Resource1._52},
            {"53", Resource1._53},
            {"54", Resource1._54},
            {"55", Resource1._55},
            {"56", Resource1._56},
            {"57", Resource1._57},
            {"58", Resource1._58},
            {"59", Resource1._59},
            {"60", Resource1._60},
            {"61", Resource1._61},
            {"62", Resource1._62},
            {"63", Resource1._63},
            {"64", Resource1._64},
            {"65", Resource1._65},
            {"66", Resource1._66},
            {"67", Resource1._67},
            {"68", Resource1._68},
            {"69", Resource1._69},
            {"70", Resource1._70},
            {"71", Resource1._71},
            {"72", Resource1._72},
            {"73", Resource1._73}
        };

        private Dictionary<int, Image[]> traxPartImages = new Dictionary<int, Image[]>()
        {
            {1, new Image[] {
                Resource1.li_1_1,
                Resource1.li_1_2,
                Resource1.li_1_3,
                Resource1.li_1_4,
                Resource1.li_1_5,
                Resource1.li_1_6,
                Resource1.li_1_7,
                Resource1.li_1_8,
                Resource1.li_1_9
            }},
            {2, new Image[] {
                Resource1.li_2_1,
                Resource1.li_2_2,
                Resource1.li_2_3,
                Resource1.li_2_4,
                Resource1.li_2_5,
                Resource1.li_2_6,
                Resource1.li_2_7,
                Resource1.li_2_8,
                Resource1.li_2_9
            }},
            {3, new Image[] {
                Resource1.li_3_1,
                Resource1.li_3_2,
                Resource1.li_3_3,
                Resource1.li_3_4,
                Resource1.li_3_5,
                Resource1.li_3_6,
                Resource1.li_3_7,
                Resource1.li_3_8,
                Resource1.li_3_9
            }},
            {4, new Image[] {
                Resource1.li_4_1,
                Resource1.li_4_2,
                Resource1.li_4_3,
                Resource1.li_4_4,
                Resource1.li_4_5,
                Resource1.li_4_6,
                Resource1.li_4_7,
                Resource1.li_4_8,
                Resource1.li_4_9
            }}
        };

        private Color[] traxColors = new Color[]
        {
            Color.FromArgb(140, 227, 1),
            Color.FromArgb(230, 179, 0),
            Color.FromArgb(218, 13, 178),
            Color.FromArgb(0, 205, 205)
        };
        #endregion

        private void txt_Title_TextChanged(object sender, EventArgs e)
        {
            if (song != null)
                song.Title = txt_Title.Text;
            ChangedSinceLastSave = true;
        }

        private void TraxMachineForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (changedSinceLastSave)
            {
                DialogResult result = MessageBox.Show(this, "Wilt u de wijzigingen die zijn aangemaakt in deze trax opslaan voordat je het programma sluit?", "Opslaan?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
                if (result == DialogResult.Yes)
                {
                    Save(string.Empty);
                }
                else if (result == System.Windows.Forms.DialogResult.Cancel)
                {
                    e.Cancel = true;
                }
            }
        }




    }
}
